function [ curvature ] = curvature( I, C, gauss )
%CURVATURE Evaluates the curvature of the contour C produced by I. I is 
%   smoothed with a gaussian filter with std dev of 'gauss'.
%   Output is a n x 1 vector.

% filter with gauss kernel
if gauss > 0
    I = imgaussfilt(I, gauss);
end

% normalized gradient
G = grad_fe(I);
G = G ./ repmat(sqrt(sum(G.^2,3)+1e-10), 1, 1, 2);
G(isnan(G)) = 0;

% divergence
D = divergence(G(:,:,1), G(:,:,2));

% divergence on contour
curvature = pw_feature(D, C);

end

