%% Exercise 5, Sheet 3
clear all,
%% LOAD IMAGES

I1 = double(imread('MPEG7/turtle-1.gif'));
I2 = double(imread('MPEG7/turtle-19.gif'));
I3 = double(imread('MPEG7/apple-20.gif'));

samples = 100;

C1 = extract_pointwise_contour(I1, samples);
C2 = extract_pointwise_contour(I2, samples);
C3 = extract_pointwise_contour(I3, samples);

%% CREATE FEATURES AND COSTS

% curvature
gauss = 5;
S1_curv = curvature(I1, C1, gauss);
S2_curv = curvature(I2, C2, gauss);
S3_curv = curvature(I3, C3, gauss);

% integral invariant
kSize = 100;
gauss = 30;
S1_intinv = integral_invariant(I1, C1, kSize, gauss);
S2_intinv = integral_invariant(I2, C2, kSize, gauss);
S3_intinv = integral_invariant(I3, C3, kSize, gauss);

% shape context
kRadius = 50;
nRadii = 10;
nAngles = 10;
S1_shcont = shape_context(I1, C1, kRadius, nRadii, nAngles);
S2_shcont = shape_context(I2, C2, kRadius, nRadii, nAngles);
S3_shcont = shape_context(I3, C3, kRadius, nRadii, nAngles);

% costs

S1toS2_curv = (repmat(S1_curv, 1, size(S2_curv,1)) - repmat(S2_curv', size(S1_curv, 1), 1)).^2;
S1toS3_curv = (repmat(S1_curv, 1, size(S3_curv,1)) - repmat(S3_curv', size(S1_curv, 1), 1)).^2;

S1toS2_intinv = (repmat(S1_intinv, 1, size(S2_intinv,1)) - repmat(S2_intinv', size(S1_intinv, 1), 1)).^2;
S1toS3_intinv = (repmat(S1_intinv, 1, size(S3_intinv,1)) - repmat(S3_intinv', size(S1_intinv, 1), 1)).^2;

S1toS2_sc = zeros(size(S1_shcont,1), size(S2_shcont,1));
% two for loops is definitely not the best way to do this... 
for i=1:size(S1_shcont,2)
   for j=1:size(S2_shcont,2)
       S1toS2_sc(i,j) = sum((S1_shcont(:,i) - S2_shcont(:,j)).^2 ./ (S1_shcont(:,i) + S2_shcont(:,j)), 1);
       if isnan(S1toS2_sc(i,j))
           S1toS2_sc(i,j) = 0;
       end
   end
end

S1toS3_sc = zeros(size(S1_shcont,1), size(S3_shcont,1));
% two for loops is definitely not the best way to do this... 
for i=1:size(S1_shcont,2)
   for j=1:size(S3_shcont,2)
       S1toS3_sc(i,j) = sum((S1_shcont(:,i) - S3_shcont(:,j)).^2 ./ (S1_shcont(:,i) + S3_shcont(:,j)), 1);
       if isnan(S1toS3_sc(i,j))
           S1toS3_sc(i,j) = 0;
       end
   end
end

%% LAPs

[curv_S1toS2, curv_12_e] = LAP(S1toS2_curv);
[curv_S1toS3, curv_13_e] = LAP(S1toS3_curv);

[intinv_S1toS2, ii_12_e] = LAP(S1toS2_intinv);
[intinv_S1toS3, ii_13_e] = LAP(S1toS3_intinv);

[sc_S1toS2, sc_12_e] = LAP(S1toS2_sc);
[sc_S1toS3, sc_13_e] = LAP(S1toS3_sc);

%% VISUALIZE

% curvature
visualise_matching(C1,C2, curv_S1toS2), suptitle(sprintf('Matching Curvature, turtle-1 to turtle-9, Energy %f', curv_12_e)),
visualise_matching(C1,C3, curv_S1toS3), suptitle(sprintf('Matching Curvature, turtle-1 to apple-20, Energy %f', curv_13_e)),

%%
% integral invariant
visualise_matching(C1,C2, intinv_S1toS2), suptitle(sprintf('Matching Integral Invariant, turtle-1 to turtle-9, Energy %f', ii_12_e)),
visualise_matching(C1,C3, intinv_S1toS3), suptitle(sprintf('Matching Integral Invariant, turtle-1 to apple-20, Energy %f', ii_13_e)),

%%
% shape context
visualise_matching(C1,C2, sc_S1toS2), suptitle(sprintf('Matching Shape Context, turtle-1 to turtle-9, Energy %f', sc_12_e)),
visualise_matching(C1,C3, sc_S1toS3), suptitle(sprintf('Matching Shape Context, turtle-1 to apple-20, Energy %f', sc_13_e)),

%% ALL FEATURES

all_costs_S1S2 = (S1toS2_curv ./ norm(S1toS2_curv,2)) + (S1toS2_intinv ./ norm(S1toS2_intinv,2)) + (S1toS2_sc ./ norm(S1toS2_sc,2));
[perm_all_turtle, e_all_turtle] = LAP(all_costs_S1S2);

all_costs_S1S3 = (S1toS3_curv ./ norm(S1toS3_curv,2)) + (S1toS3_intinv ./ norm(S1toS3_intinv,2)) + (S1toS3_sc ./ norm(S1toS3_sc,2));
[perm_all_apple, e_all_apple] = LAP(all_costs_S1S3);

% all features
visualise_matching(C1,C2, perm_all_turtle), suptitle(sprintf('Matching All Features, turtle-1 to turtle-9, Energy %f', e_all_turtle)),
visualise_matching(C1,C3, perm_all_apple), suptitle(sprintf('Matching All Features, turtle-1 to apple-20, Energy %f', e_all_apple)),
